@@ -2,6 +2,7 @@
 #'
 #' This function maps raster layer values.
 #' A key showing how the colors map to raster values is shown below the map.
+#' The width and height of the graphics region will be automagically determined in some cases.
 #'
 #' @param r RasterLayer, SpatialGridDataFrame, or CRS.
 #'   A raster layer with values to be plotted or a coordinate reference system (CRS).
@@ -50,6 +51,7 @@
 #'   Vector of length 2 giving the maximum width and height for the graphics device in picas, respectively.
 #'   Suggested dimensions for single-column, double-column, and sidetitle figures are
 #'   \code{c(21, 56)}, \code{c(43, 56)}, and \code{c(56, 43)}, respectively.
+#'   This argument is only applicable when the \code{file} argument is specified.
 #' @param labels list.
 #'   Describes the location and values of labels in the color key.
 #'   This list may include components \code{at} and \code{labels}.
@@ -60,9 +62,10 @@
 #'   Position of the north arrow:
 #'   "bottomleft", "topleft", "topright", or "bottomright" to denote arrow location.
 #' @param explanation character.
-#'   Label explaining the raster value.
+#'   Label explaining the raster cell value.
 #' @param credit character.
-#'   Label crediting the base map.
+#'   Label crediting the base map;
+#'   by default, the character string describing the raster layer's (\code{r}) projection and datum in the PROJ.4 format.
 #' @param shade list.
 #'   If specified, a semi-transparent shade layer is drawn on top of the raster layer.
 #'   This layer is described using a list of arguments supplied to \code{raster::hillShade} function.
@@ -90,12 +93,19 @@
 #'   If true, a color key should be drawn.
 #' @param draw.raster logical.
 #'   If true, the raster image is drawn.
+#' @param file character.
+#'   Name of the output file.
+#'   Specifying this argument will start a graphics device driver for producing a
+#'   PDF or PNG file format---the file extension determines the format type.
+#'   The width and height of the graphics region will be automagically determined and
+#'   included with the function's returned values, see "Value" section for details.
+#'   These graphical parameters can be used when creating similar map layouts in dynamic reports
+#'   or whenever further elements will be added to an existing plot generated by this function
+#'   (possible when \code{file = NULL}).
 #' @param useRaster logical.
 #'   If true, a bitmap raster is used to plot \code{r} instead of using polygons.
 #'   If \code{UseRaster} is not specified, raster images are used when the \code{getOption("preferRaster")} is true.
 #'
-#' @details The dimensions of a new graphics device is dependent on the argument values of \code{max.dev.dim} and \code{asp}.
-#'
 #' @return Used for the side-effect of a new plot generated.
 #'   Returns a \code{list} object with the following graphical parameters:
 #'   \describe{
@@ -117,23 +127,40 @@
 #' @export
 #'
 #' @examples
-#' library(raster)
-#'
-#' r <- raster(nrow = 10, ncol = 10)
+#' r <- raster::raster(nrow = 10, ncol = 10)
 #' r[] <- 1L
 #' r[51:100] <- 2L
 #' r[3:6, 1:5] <- 8L
-#' r <- ratify(r)
-#' rat <- levels(r)[[1]]
+#' r <- raster::ratify(r)
+#' rat <- raster::levels(r)[[1]]
 #' rat$land.cover <- c("Pine", "Oak", "Meadow")
 #' rat$code <- c(12, 25, 30)
 #' levels(r) <- rat
 #' PlotMap(r, att = "land.cover", col = c("grey", "orange", "purple"))
 #' PlotMap(r, att = "code")
 #'
+#' r <- raster::raster(system.file("external/test.grd", package="raster"))
+#' credit <- "Label crediting the base map."
+#' explanation <- "Label explaining the raster cell value."
+#' PlotMap(r, scale.loc = "topleft", dms.tick = TRUE, trim.r = TRUE,
+#'         credit = credit, explanation = explanation)
+#' data(meuse, package = "sp")
+#' sp::coordinates(meuse) = ~ x + y
+#' points(meuse)
+#'
+#' val <- PlotMap(r, scale.loc = "topleft", dms.tick = TRUE, trim.r = TRUE,
+#'                credit = credit, explanation = explanation,
+#'                file = "Rplots1.pdf")
+#' print(val)
+#'
+#' pdf(file = "Rplots2.pdf", width = val$din[1], height = val$din[2])
+#' PlotMap(r, scale.loc = "topleft", dms.tick = TRUE, trim.r = TRUE,
+#'         credit = credit, explanation = explanation)
+#' points(meuse)
+#' dev.off()
+#'
+#' file.remove(c("Rplots1.pdf", "Rplots2.pdf"))
 #' graphics.off()
-#' r <- raster(system.file("external/test.grd", package="raster"))
-#' PlotMap(r, scale.loc = "topleft", dms.tick = TRUE, trim.r = TRUE)
 #'
 
 PlotMap <- function(r, layer=1, att=NULL, n, breaks, xlim=NULL, ylim=NULL,
@@ -143,7 +170,8 @@ PlotMap <- function(r, layer=1, att=NULL, n, breaks, xlim=NULL, ylim=NULL,
                     max.dev.dim=c(43, 56), labels=NULL, scale.loc="bottomleft",
                     arrow.loc=NULL, explanation=NULL, credit=proj4string(r),
                     shade=NULL, contour.lines=NULL, rivers=NULL, lakes=NULL,
-                    roads=NULL, draw.key=NULL, draw.raster=TRUE, useRaster) {
+                    roads=NULL, draw.key=NULL, draw.raster=TRUE, file=NULL,
+                    useRaster) {
 
   if (!is.null(bg.image) && !inherits(bg.image, "RasterLayer"))
     stop("background image is the incorrect class")
@@ -332,7 +360,8 @@ PlotMap <- function(r, layer=1, att=NULL, n, breaks, xlim=NULL, ylim=NULL,
     mar1 <- c(0, 0, 0, 0)
   }
 
-  if (grDevices::dev.cur() > 1) {
+  if (is.null(file)) {
+    if (grDevices::dev.cur() == 1) grDevices::dev.new()
     dev.dim <- grDevices::dev.size() / inches.in.pica
     w <- dev.dim[1]
     h1 <- y1 + mar1[1] + mar1[3]
@@ -345,14 +374,19 @@ PlotMap <- function(r, layer=1, att=NULL, n, breaks, xlim=NULL, ylim=NULL,
       h2 <- y2 + mar2[1] + mar2[3]
       h1 <- y1 + mar1[1] + mar1[3]
       h <- h1 + h2
-      if (h > max.dev.dim[2])
-        w <- w - 0.01
-      else
-        break
+      if (h > max.dev.dim[2]) w <- w - 0.01 else break
     }
     wi <- w * inches.in.pica
     hi <- h * inches.in.pica
-    grDevices::dev.new(width=wi, height=hi)
+    ext <- tolower(tools::file_ext(file))
+    if (ext == "pdf") {
+      grDevices::pdf(file, width=wi, height=hi)
+    } else if (ext == "png") {
+      grDevices::png(file, width=wi * 100, height=hi * 100, res=100)
+    } else {
+      stop("file argument does not have a valid extension")
+    }
+    on.exit(grDevices::dev.off())
   }
 
   if (draw.key)
@@ -525,7 +559,9 @@ PlotMap <- function(r, layer=1, att=NULL, n, breaks, xlim=NULL, ylim=NULL,
   if (!is.null(arrow.loc))
     .AddNorthArrow(arrow.loc, r@crs, cex)
 
-  invisible(list(din=graphics::par("din"), usr=graphics::par("usr"), heights=c(h2, h1) / h))
+  invisible(list(din=graphics::par("din"),
+                 usr=graphics::par("usr"),
+                 heights=c(h2, h1) / h))
 }
 
 
@@ -604,3 +640,4 @@ PlotMap <- function(r, layer=1, att=NULL, n, breaks, xlim=NULL, ylim=NULL,
   sapply(x, function(i) tryCatch(is.matrix(grDevices::col2rgb(i)),
                                  error=function(e) FALSE))
 }
+
